from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="CharacterizationStatistic")


@_attrs_define
class CharacterizationStatistic:
    """CharacterizationStatistic.

    This provides the statistics for a specific design space.

        Attributes:
            min_ (float):
            estimate (float):
            max_ (float):
    """

    min_: float
    estimate: float
    max_: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        min_ = self.min_

        estimate = self.estimate

        max_ = self.max_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "min": min_,
            "estimate": estimate,
            "max": max_,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        min_ = d.pop("min")

        estimate = d.pop("estimate")

        max_ = d.pop("max")

        characterization_statistic = cls(
            min_=min_,
            estimate=estimate,
            max_=max_,
        )

        characterization_statistic.additional_properties = d
        return characterization_statistic

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
